<?php

namespace App\DataTables\Reports;

use App\Models\Invoice;
use Illuminate\Database\Eloquent\Builder as QueryBuilder;
use Yajra\DataTables\EloquentDataTable;
use Yajra\DataTables\Html\Builder as HtmlBuilder;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Html\Column;
use Yajra\DataTables\Html\Editor\Editor;
use Yajra\DataTables\Html\Editor\Fields;
use Yajra\DataTables\Services\DataTable;
use DateTime;
use Carbon\Carbon;
use App\Facades\Company;

class CreditNoteRegister extends DataTable
{
    /**
     * Build the DataTable class.
     *
     * @param QueryBuilder $query Results from query() method.
     */
    public function dataTable(QueryBuilder $query): EloquentDataTable
    {
        return (new EloquentDataTable($query))


            ->addColumn('month', function ($query) {
                return date('F', strtotime($query->invoicedate));
            })
            ->filterColumn('month', function ($query, $keyword) {
                // Parse the month name to a date format
                $monthDate = date_parse($keyword);

                if ($monthDate['error_count'] === 0 && $monthDate['warning_count'] === 0) {
                    $query->whereRaw("MONTH(invoicedate) = ?", [$monthDate['month']]);
                }
            })->addColumn('customer', function ($query) {
                return $query->customer ? $query->customer->company : '';
            })->filterColumn('customer', function ($query, $keyword) {
                $query->whereHas('customer', function ($subquery) use ($keyword) {
                    $subquery->where('company', 'like', "%{$keyword}%");
                });
            })


            ->addColumn('invoice_id', function ($query) {
                return $query->company ? $query->company->companyPrefix . '' . $query->parent->saleorder_id . '-' . $query->parent->invoice_serial : '';
            })
            ->addColumn('cndate', function ($query) {
                return date('d M y', strtotime($query->invoicedate));
            })
            ->addColumn('cndate', function ($query) {
                return date('d M y', strtotime($query->invoicedate));
            })
            ->addColumn('cn_id', function ($query) {
                return $query->company ? 'C' . $query->company->companyPrefix . '' . $query->saleorder_id . '-' . $query->invoice_serial : '';
            })->filterColumn('cn_id', function ($query, $keyword) {
                $query->orWhere('saleorder_id', 'like', '%' . $keyword . '%');
            })
            ->addColumn('totalAmount', function ($query) {
                $viewTotal = $query->getTotalAmountSumAttribute() + $query->getTotalChargeSumAttribute();
                return $query->company ? $query->company->companyCurrency . '' . number_format($viewTotal, 2) : '';
            })
            ->addColumn('reasonOfReturn', function ($query) {
                return "NA";
            })
            ->addColumn('amountBeforeAdjustment', function ($query) {
                $viewTotal = $query->parent->getTotalAmountSumAttribute() + $query->parent->getTotalChargeSumAttribute();
                return $query->company ? $query->company->companyCurrency . '' . number_format($viewTotal, 2) : '';
            })

            ->addColumn('amountAfterAdjustment', function ($query) {
                $inv = $query->parent->getTotalAmountSumAttribute() + $query->parent->getTotalChargeSumAttribute();
                $memo = $query->getTotalAmountSumAttribute() + $query->getTotalChargeSumAttribute();
                return $query->company ? $query->company->companyCurrency . '' . number_format($inv - $memo, 2) : '';
            })
            ->setRowId('id');
    }

    /**
     * Get the query source of dataTable.
     */
    public function query(Invoice $model): QueryBuilder
    {
        //return $model->newQuery()->latest();

        return $model->newQuery()
            ->where('order_type', 2)
            ->where(function ($query) {
                $fromDate = $this->request->input('fromdate');
                $toDate = $this->request->input('todate');

                if ($fromDate) {
                    $fromDate = Carbon::parse($fromDate)->startOfDay();
                    $query->where('invoicedate', '>=', $fromDate);
                }

                if ($toDate) {
                    $toDate = Carbon::parse($toDate)->endOfDay();
                    $query->where('invoicedate', '<=', $toDate);
                }
            })
            ->where('company_id', auth()->user()->company_id)
            ->latest();



    }

    /**
     * Optional method if you want to use the html builder.
     */
    public function html(): HtmlBuilder
    {
        return $this->builder()
            ->setTableId('orders-table')
            ->columns([
                'month' => ['title' => 'Month'],
                'customer' => ['title' => 'Customer'],
                'invoicedate' => ['title' => 'Invoice Date'],
                'invoice_id' => ['title' => 'Invoice #'],
                'cndate' => ['title' => 'CN Date'],
                'cn_id' => ['title' => 'CN #'],
                'reasonOfReturn' => ['title' => 'Reason of Return'],
                'amountBeforeAdjustment' => ['title' => 'Amount before CN Adjustment'],
                'totalAmount' => ['title' => 'Adjustment Amount'],
                'amountAfterAdjustment' => ['title' => 'Amount after Adjustment']
            ])
            ->minifiedAjax('', null, request()->only(['fromdate', 'todate']))
            //->dom('Bfrtip')
            ->orderBy(1)
            ->selectStyleSingle()
            ->buttons([
                Button::make('excel'),
                Button::make('csv'),
                Button::make('pdf'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            ]);
    }

    /**
     * Get the dataTable columns definition.
     */



    public function getColumns(): array
    {
        return [

            'month',
            'customer',
            'invoicedate',
            'saleorder_id',
            'cndate',
            'cn_id',
            'totalAmount',
            'reasonOfReturn',
            'amountBeforeAdjustment',
            'adjustmentAmount',
            'amountAfterAdjustment'

        ];
    }

    /**
     * Get the filename for export.
     */
    protected function filename(): string
    {
        return 'Invoices_' . date('YmdHis');
    }
}
