<?php

namespace App\DataTables\Reports;

use App\Models\PurchaseItem;
use Illuminate\Database\Eloquent\Builder as QueryBuilder;
use Yajra\DataTables\EloquentDataTable;
use Yajra\DataTables\Html\Builder as HtmlBuilder;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Html\Column;
use Yajra\DataTables\Html\Editor\Editor;
use Yajra\DataTables\Html\Editor\Fields;
use Yajra\DataTables\Services\DataTable;
use DateTime;
use Carbon\Carbon;
use App\Facades\Company;

class InventoryAgingReport extends DataTable
{
    /**
     * Build the DataTable class.
     *
     * @param QueryBuilder $query Results from query() method.
     */



    public function dataTable(QueryBuilder $query): EloquentDataTable
    {
        return (new EloquentDataTable($query))
            ->addColumn('name', function ($query) {
                return $query->orderItem->name;
            })
            ->addColumn('vendor', function ($query) {
                return $query->purchase->vendor->company;
            })
            ->addColumn('one_thirty', function ($query) {
                return $this->categorizeDate($query->created_at, 1);
            })
            ->addColumn('thirtyone_sixty', function ($query) {
                return $this->categorizeDate($query->created_at, 2);
            })
            ->addColumn('sixtyone_ninety', function ($query) {
                return $this->categorizeDate($query->created_at, 3);
            })
            ->addColumn('ninetyone_onetwnty', function ($query) {
                return $this->categorizeDate($query->created_at, 4);
            })
            ->addColumn('onetwntyone_so_on', function ($query) {
                return $this->categorizeDate($query->created_at, 5);
            })
            ->setRowId('id');
    }

    private function categorizeDate($date, $num)
    {
        $thetick = '✔';
        $currentDate = now();
        $difference = $currentDate->diff($date);
        $daysDifference = $difference->days;

        if ($daysDifference >= 1 && $daysDifference <= 30) {

            return ($num == 1) ? $thetick : '-';

        } elseif ($daysDifference >= 31 && $daysDifference <= 60) {
            return ($num == 2) ? $thetick : '-';
        } elseif ($daysDifference >= 61 && $daysDifference <= 90) {
            return ($num == 3) ? $thetick : '-';
        } elseif ($daysDifference >= 91 && $daysDifference <= 120) {
            return ($num == 4) ? $thetick : '-';
        } else {
            return ($num == 5) ? $thetick : '-';
        }
    }
    /**
     * Get the query source of dataTable.
     */
    public function query(PurchaseItem $model): QueryBuilder
    {
        //return $model->newQuery()->latest();

        return $model->newQuery()

            ->where(function ($query) {
                $fromDate = $this->request->input('fromdate');
                $toDate = $this->request->input('todate');

                if ($fromDate) {
                    $fromDate = Carbon::parse($fromDate)->startOfDay();
                    $query->where('created_at', '>=', $fromDate);
                }

                if ($toDate) {
                    $toDate = Carbon::parse($toDate)->endOfDay();
                    $query->where('created_at', '<=', $toDate);
                }
            })
            ->whereDoesntHave('invoiceItem')
            ->latest();



    }

    /**
     * Optional method if you want to use the html builder.
     */
    public function html(): HtmlBuilder
    {
        return $this->builder()
            ->setTableId('orders-table')
            ->columns([
                'name' => ['title' => 'Item Name'],
                'vendor' => ['title' => 'Supplier'],
                //  'created_at' => ['title' => 'Invoice Date'],
                'one_thirty' => ['title' => '1 to 30 days'],
                'thirtyone_sixty' => ['title' => '31 to 60 days'],
                'sixtyone_ninety' => ['title' => '61 to 90 days'],
                'ninetyone_onetwnty' => ['title' => '91 to 120 days'],
                'onetwntyone_so_on' => ['title' => 'Over 121 days'],
            ])
            ->minifiedAjax('', null, request()->only(['fromdate', 'todate']))
            //->dom('Bfrtip')
            ->orderBy(1)
            ->selectStyleSingle()
            ->buttons([
                Button::make('excel'),
                Button::make('csv'),
                Button::make('pdf'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            ]);
    }

    /**
     * Get the dataTable columns definition.
     */



    public function getColumns(): array
    {
        return [
            'name',
            'vendor',
            'one_thirty',
            'thirtyone_sixty',
            'sixtyone_ninety',
            'ninetyone_onetwnty',
            'onetwntyone_so_on',
        ];
    }

    /**
     * Get the filename for export.
     */
    protected function filename(): string
    {
        return 'InventoryAging_' . date('YmdHis');
    }
}
