<?php

namespace App\DataTables\Reports;

use App\Models\Order;
use App\Models\Purchase;
use Illuminate\Database\Eloquent\Builder as QueryBuilder;
use Yajra\DataTables\EloquentDataTable;
use Yajra\DataTables\Html\Builder as HtmlBuilder;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Html\Column;
use Yajra\DataTables\Html\Editor\Editor;
use Yajra\DataTables\Html\Editor\Fields;
use Yajra\DataTables\Services\DataTable;
use DateTime;
use Carbon\Carbon;

class PendingOrderReport extends DataTable
{
    /**
     * Build the DataTable class.
     *
     * @param QueryBuilder $query Results from query() method.
     */
    public function dataTable(QueryBuilder $query): EloquentDataTable
    {
        $counter = 0;
        return (new EloquentDataTable($query))
            ->addIndexColumn()

            ->addColumn('serial_no', function () use (&$counter) {
                // Increment the counter for each row
                return ++$counter;
            })
            ->addColumn('salesRep', function ($query) {
                return $query->saleRep ? $query->saleRep->name : '';
            })
            ->filterColumn('salesRep', function ($query, $keyword) {
                $query->whereHas('saleRep', function ($subquery) use ($keyword) {
                    $subquery->where('name', 'like', "%{$keyword}%");
                });
            })

            ->addColumn('month', function ($query) {
                return date('F', strtotime($query->orderdate));
            })
            ->filterColumn('month', function ($query, $keyword) {
                // Parse the month name to a date format
                $monthDate = date_parse($keyword);

                if ($monthDate['error_count'] === 0 && $monthDate['warning_count'] === 0) {
                    $query->whereRaw("MONTH(orderdate) = ?", [$monthDate['month']]);
                }
            })

            ->addColumn('orderdate', function ($query) {
                return date('d M y', strtotime($query->orderdate));
            })
            ->addColumn('orderdate', function ($query) {
                return date('d M y', strtotime($query->orderdate));
            })
            ->addColumn('daysToOrder', function ($query) {
                $orderdate = new DateTime($query->orderdate);
                $today = new DateTime();
                $interval = $today->diff($orderdate);
                $daysToOrder = $interval->days;
                return $orderdate <= $today ? -$daysToOrder : $daysToOrder;
            })
            ->filterColumn('daysToOrder', function ($query, $keyword) {
                $keyword = intval($keyword); // Convert keyword to an integer
    
                // Use WHERE clause to search for both positive and negative values
                $query->whereRaw('DATEDIFF(orderdate, CURDATE()) = ? OR DATEDIFF(orderdate, CURDATE()) = ?', [$keyword, -$keyword]);
            })


            ->addColumn('customer', function ($query) {
                return $query->customer ? $query->customer->company : '';
            })
            ->filterColumn('customer', function ($query, $keyword) {
                $query->whereHas('customer', function ($subquery) use ($keyword) {
                    $subquery->where('company', 'like', "%{$keyword}%");
                });
            })->addColumn('saleorder_id', function ($query) {
                return $query->company ? $query->company->companyPrefix . '' . $query->saleorder_id : '';
            })->filterColumn('saleorder_id', function ($query, $keyword) {
                $query->orWhere('saleorder_id', 'like', '%' . $keyword . '%');
            })

            ->addColumn('totalAmount', function ($query) {

                $viewTotal = $query->getTotalAmountSumAttribute() + $query->getTotalChargeSumAttribute() - $query->purchaseItemPurchasedTillNow();
                return $query->company ? $query->company->companyCurrency . '' . number_format($viewTotal, 2) : '';
            })
            ->setRowId('id');
    }

    /**
     * Get the query source of dataTable.
     */
    public function query(Order $model): QueryBuilder
    {




        return $model->newQuery()
            ->where('status_id', 1)
            ->where(function ($query) {
                $fromDate = $this->request->input('fromdate');
                $toDate = $this->request->input('todate');

                if ($fromDate) {
                    $fromDate = Carbon::parse($fromDate)->startOfDay();
                    $query->where('orderdate', '>=', $fromDate);
                }

                if ($toDate) {
                    $toDate = Carbon::parse($toDate)->endOfDay();
                    $query->where('orderdate', '<=', $toDate);
                }
            })
            ->latest();


    }

    /**
     * Optional method if you want to use the html builder.
     */
    public function html(): HtmlBuilder
    {
        return $this->builder()
            ->setTableId('orders-table')
            ->columns([
                'serial_no' => ['title' => '#'],

                'month' => ['title' => 'Month'],
                'salesRep' => ['title' => 'Sales Rep'],
                'orderdate' => ['title' => 'SO Date'],
                'saleorder_id' => ['title' => 'SO#'],

                'customer' => ['title' => 'Customer Name'],
                'ref' => ['title' => 'PO Reference'],
                'totalAmount' => ['title' => 'Total SO Amount'],
                'daysToOrder' => ['title' => 'Days of Pending'],


            ])
            ->minifiedAjax('', null, request()->only(['fromdate', 'todate']))
            //->dom('Bfrtip')
            ->orderBy(1)
            ->selectStyleSingle()
            ->buttons([
                Button::make('excel'),
                Button::make('csv'),
                Button::make('pdf'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            ]);
    }

    /**
     * Get the dataTable columns definition.
     */



    public function getColumns(): array
    {
        return [

            'serial_no',
            'salesRep',
            'month',
            'ref',
            'customer',
            'orderdate',
            'saleorder_id',
            'totalAmount',
            'daysToOrder',



        ];
    }

    /**
     * Get the filename for export.
     */
    protected function filename(): string
    {
        return 'Orders_' . date('YmdHis');
    }
}
