<?php

namespace App\DataTables\Reports;

use App\Models\Quotation;
use Illuminate\Database\Eloquent\Builder as QueryBuilder;
use Yajra\DataTables\EloquentDataTable;
use Yajra\DataTables\Html\Builder as HtmlBuilder;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Html\Column;
use Yajra\DataTables\Html\Editor\Editor;
use Yajra\DataTables\Html\Editor\Fields;
use Yajra\DataTables\Services\DataTable;
use DateTime;
use Carbon\Carbon;

class PendingQuotationReport extends DataTable
{
    /**
     * Build the DataTable class.
     *
     * @param QueryBuilder $query Results from query() method.
     */


    public function dataTable(QueryBuilder $query): EloquentDataTable
    {

        $counter = 0;

        return (new EloquentDataTable($query))
            ->addIndexColumn()

            ->addColumn('serial_no', function () use (&$counter) {
                // Increment the counter for each row
                return ++$counter;
            })
            ->addColumn('salesRep', function ($query) {
                return $query->saleRep ? $query->saleRep->name : '';
            })
            ->filterColumn('salesRep', function ($query, $keyword) {
                $query->whereHas('saleRep', function ($subquery) use ($keyword) {
                    $subquery->where('name', 'like', "%{$keyword}%");
                });
            })

            ->addColumn('month', function ($query) {
                return date('F', strtotime($query->quotedate));
            })
            ->filterColumn('month', function ($query, $keyword) {
                // Parse the month name to a date format
                $monthDate = date_parse($keyword);

                if ($monthDate['error_count'] === 0 && $monthDate['warning_count'] === 0) {
                    $query->whereRaw("MONTH(quotedate) = ?", [$monthDate['month']]);
                }
            })

            ->addColumn('quotedate', function ($query) {
                return date('d M y', strtotime($query->quotedate));
            })

            ->addColumn('daysToQuote', function ($query) {
                $quotedate = new DateTime($query->quotedate);
                $today = new DateTime();
                $interval = $today->diff($quotedate);
                $daysToQuote = $interval->days;
                return $quotedate <= $today ? -$daysToQuote : $daysToQuote;
            })
            ->filterColumn('daysToQuote', function ($query, $keyword) {
                $keyword = intval($keyword); // Convert keyword to an integer
    
                // Use WHERE clause to search for both positive and negative values
                $query->whereRaw('DATEDIFF(quotedate, CURDATE()) = ? OR DATEDIFF(quotedate, CURDATE()) = ?', [$keyword, -$keyword]);
            })


            ->addColumn('customer', function ($query) {
                return $query->customer ? $query->customer->company : '';
            })
            ->filterColumn('customer', function ($query, $keyword) {
                $query->whereHas('customer', function ($subquery) use ($keyword) {
                    $subquery->where('company', 'like', "%{$keyword}%");
                });
            })
            // ->addColumn('amend', function ($query) {

            //     return "";
            // })
            ->addColumn('totalAmount', function ($query) {
                $viewTotal = $query->getTotalAmountSumAttribute() + $query->getTotalChargeSumAttribute();
                return $query->company ? $query->company->companyCurrency . '' . number_format($viewTotal, 2) : '';
            })
            ->setRowId('id');
    }

    /**
     * Get the query source of dataTable.
     */
    public function query(Quotation $model): QueryBuilder
    {
        //return $model->newQuery()->latest();

        return $model->newQuery()
            ->where('orderType', 0)
            ->where(function ($query) {
                $fromDate = $this->request->input('fromdate');
                $toDate = $this->request->input('todate');

                if ($fromDate) {
                    $fromDate = Carbon::parse($fromDate)->startOfDay();
                    $query->where('quotedate', '>=', $fromDate);
                }

                if ($toDate) {
                    $toDate = Carbon::parse($toDate)->endOfDay();
                    $query->where('quotedate', '<=', $toDate);
                }
            })
            ->latest();



    }

    /**
     * Optional method if you want to use the html builder.
     */
    public function html(): HtmlBuilder
    {
        return $this->builder()
            ->setTableId('quotes-table')
            ->columns([
                'serial_no' => ['title' => '#'],
                'salesRep' => ['title' => 'Sales Rep'],
                'month' => ['title' => 'Month'],
                'quotedate' => ['title' => 'Quotation Date'],
                'quoteid' => ['title' => 'Quotation No'],
                'customer' => ['title' => 'Customer'],
                'ref' => ['title' => 'PO Reference'],

                'totalAmount' => ['title' => 'Total Quotations Amount'],
                'daysToQuote' => ['title' => 'No of days'],
              //  'amend' => ['title' => 'Original Amended'],

            ])
            ->minifiedAjax('', null, request()->only(['fromdate', 'todate']))
            //->dom('Bfrtip')
            ->orderBy(1)
            ->selectStyleSingle()
            ->buttons([
                Button::make('excel'),
                Button::make('csv'),
                Button::make('pdf'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            ]);
    }

    /**
     * Get the dataTable columns definition.
     */



    public function getColumns(): array
    {
        return [

            'serial_no',
            'salesRep',
            'month',
            'ref',
            'customer',
            'quotedate',
            'quoteid',
            'totalAmount',
            'daysToQuote',
            'action',


        ];
    }

    /**
     * Get the filename for export.
     */
    protected function filename(): string
    {
        return 'Quotes_' . date('YmdHis');
    }
}
