<?php

namespace App\DataTables\Reports\Purchase;

use App\Models\PurchaseInvoice;
use Illuminate\Database\Eloquent\Builder as QueryBuilder;
use Yajra\DataTables\EloquentDataTable;
use Yajra\DataTables\Html\Builder as HtmlBuilder;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Html\Column;
use Yajra\DataTables\Html\Editor\Editor;
use Yajra\DataTables\Html\Editor\Fields;
use Yajra\DataTables\Services\DataTable;
use DateTime;
use Carbon\Carbon;
use App\Facades\Company;

class PurchaseRegister extends DataTable
{
    /**
     * Build the DataTable class.
     *
     * @param QueryBuilder $query Results from query() method.
     */
    public function dataTable(QueryBuilder $query): EloquentDataTable
    {
        return (new EloquentDataTable($query))

            ->addColumn('month', function ($query) {
                return date('F', strtotime($query->invoicedate));
            })
            ->filterColumn('month', function ($query, $keyword) {
                // Parse the month name to a date format
                $monthDate = date_parse($keyword);

                if ($monthDate['error_count'] === 0 && $monthDate['warning_count'] === 0) {
                    $query->whereRaw("MONTH(invoicedate) = ?", [$monthDate['month']]);
                }
            })

            ->addColumn('invoicedate', function ($query) {
                return date('d M y', strtotime($query->invoicedate));
            })
            ->addColumn('invoicedate', function ($query) {
                return date('d M y', strtotime($query->invoicedate));
            })
            ->addColumn('vendor', function ($query) {
                return $query->vendor ? $query->vendor->company : '';
            })->filterColumn('vendor', function ($query, $keyword) {
            $query->whereHas('vendor', function ($subquery) use ($keyword) {
                $subquery->where('company', 'like', "%{$keyword}%");
            });
        })->addColumn('saleorder_id', function ($query) {
            $alphabet = array('', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z');

            return $query->company ? $query->company->companyPrefix . '' . $query->saleorder_id . $alphabet[$query->purchaseGroup] . '-' . $query->invoice_serial : '';
        })->filterColumn('saleorder_id', function ($query, $keyword) {
            $query->orWhere('saleorder_id', 'like', '%' . $keyword . '%');
        })
            ->addColumn('subtotal', function ($query) {
                $viewTotal = $query->getTotalAmountSumAttribute();
                return $query->company ? $query->company->companyCurrency . '' . number_format($viewTotal, 2) : '';
            })
            ->addColumn('freight', function ($query) {
                $viewTotal = $query->getTotalShippingSumAttribute();
                return $query->company ? $query->company->companyCurrency . '' . number_format($viewTotal, 2) : '';
            })
            ->addColumn('cccharges', function ($query) {
                $viewTotal = $query->getTotalCreditCardSumAttribute();
                return $query->company ? $query->company->companyCurrency . '' . number_format($viewTotal, 2) : '';
            })
            ->addColumn('tax', function ($query) {
                $viewTotal = $query->getTotalTaxSumAttribute();
                return $query->company ? $query->company->companyCurrency . '' . number_format($viewTotal, 2) : '';
            })
            ->addColumn('totalAmount', function ($query) {
                $viewTotal = $query->getTotalAmountSumAttribute() + $query->getTotalChargeSumAttribute();
                return $query->company ? $query->company->companyCurrency . '' . number_format($viewTotal, 2) : '';
            })
            ->addColumn('zipCode', function ($query) {

                return $query->shippingPostal;
            })
            ->addColumn('state', function ($query) {

                return $query->state ? $query->state->name : '';
            })

            ->addColumn('status', function ($query) {

                return 'NA';
            })
            ->addColumn('tracking', function ($query) {

                return 'NA';
            })
            ->setRowId('id');
    }

    /**
     * Get the query source of dataTable.
     */
    public function query(PurchaseInvoice $model): QueryBuilder
    {
        //return $model->newQuery()->latest();

        return $model->newQuery()
            ->where('order_type', 1)
            ->where(function ($query) {
                $fromDate = $this->request->input('fromdate');
                $toDate = $this->request->input('todate');

                if ($fromDate) {
                    $fromDate = Carbon::parse($fromDate)->startOfDay();
                    $query->where('invoicedate', '>=', $fromDate);
                }

                if ($toDate) {
                    $toDate = Carbon::parse($toDate)->endOfDay();
                    $query->where('invoicedate', '<=', $toDate);
                }
            })
            ->latest();



    }

    /**
     * Optional method if you want to use the html builder.
     */
    public function html(): HtmlBuilder
    {
        return $this->builder()
            ->setTableId('orders-table')
            ->columns([
                'month' => ['title' => 'Month'],
                'invoicedate' => ['title' => 'Invoice Date'],
                'saleorder_id' => ['title' => 'Invoice #'],
                'vendor' => ['title' => 'Vendor'],
                'ref' => ['title' => 'PO Reference'],
                'subtotal' => ['title' => 'Gross Amount'],
                'freight' => ['title' => 'Freight'],
                'cccharges' => ['title' => 'CC Charges'],
                'tax' => ['title' => 'Tax'],
                'totalAmount' => ['title' => 'Total Invoice Amount'],
                'zipCode' => ['title' => 'Zip/Postal Code'],
                'state' => ['title' => 'State'],

                'status' => ['title' => 'Status'],
                'tracking' => ['title' => 'Tracking'],


            ])
            ->minifiedAjax('', null, request()->only(['fromdate', 'todate']))
            //->dom('Bfrtip')
            ->orderBy(1)
            ->selectStyleSingle()
            ->buttons([
                Button::make('excel'),
                Button::make('csv'),
                Button::make('pdf'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            ]);
    }

    /**
     * Get the dataTable columns definition.
     */



    public function getColumns(): array
    {
        return [


            'salesRep',
            'month',
            'ref',
            'vendor',
            'invoicedate',
            'saleorder_id',
            'subtotal',
            'freight',
            'cccharges',
            'tax',
            'totalAmount',
            'zipCode',
            'state',
            'country',


        ];
    }

    /**
     * Get the filename for export.
     */
    protected function filename(): string
    {
        return 'Invoices_' . date('YmdHis');
    }
}
